-------------------------------------------------------------------------------
-- Title      : firUnit
-- Project    : 
-------------------------------------------------------------------------------
-- File       : operativeUnit.vhd
-- Author     : Jean-Noel BAZIN  <jnbazin@pc-disi-026.enst-bretagne.fr>
-- Company    : 
-- Created    : 2018-04-11
-- Last update: 2018-04-11
-- Platform   : 
-- Standard   : VHDL'93/02
-------------------------------------------------------------------------------
-- Description: 8 bit FIR
-------------------------------------------------------------------------------
-- Copyright (c) 2018 
-------------------------------------------------------------------------------
-- Revisions  :
-- Date        Version  Author  Description
-- 2018-04-11  1.0      jnbazin Created
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity firUnit is

  port (
    I_clock               : in  std_logic;  -- global clock
    I_reset               : in  std_logic;  -- asynchronous global reset
    I_inputSample         : in  std_logic_vector(10 downto 0);  -- 11 bit input sample (dont 1 bit de signe)
    I_inputSampleValid    : in  std_logic;
    O_filteredSample      : out std_logic_vector(10 downto 0);  -- filtered sample
    O_filteredSampleValid : out std_logic
    );

end entity firUnit;

architecture archi_firUnit of firUnit is

  component controlUnit is
    port (
      I_clock               : in  std_logic;
      I_reset               : in  std_logic;
      I_inputSampleValid    : in  std_logic;
      I_processingDone_1      : in  std_logic;
      I_processingDone_2a      : in  std_logic;
      I_processingDone_2b      : in  std_logic;
      I_processingDone_3      : in  std_logic;
      O_loadShift_1         : out std_logic;  -- filtered sample
      O_loadShift_2b        : out std_logic;  -- filtered sample
      O_loadShift_2a        : out std_logic;  -- filtered sample
      O_loadShift_3         : out std_logic;  -- filtered sample
      O_initAddress         : out std_logic;
      O_incrAddress         : out std_logic;
      O_initSum             : out std_logic;
      O_loadSum             : out std_logic;
      O_loadY               : out std_logic;
      O_FilteredSampleValid : out std_logic);
  end component controlUnit;

  component operativeUnit is
    port (
      I_clock          : in  std_logic;
      I_reset          : in  std_logic;
      I_inputSample    : in  std_logic_vector(10 downto 0);  -- 11 bit input sample (dont 1 bit de signe)
      I_loadShift_1         : in std_logic;  -- filtered sample
      I_loadShift_2b        : in std_logic;  -- filtered sample
      I_loadShift_2a        : in std_logic;  -- filtered sample
      I_loadShift_3    : in std_logic;  -- filtered sample
      I_initAddress    : in  std_logic;
      I_incrAddress    : in  std_logic;
      I_initSum        : in  std_logic;
      I_loadSum        : in  std_logic;
      I_loadY          : in  std_logic;
      O_processingDone_1 : out std_logic;
      O_processingDone_2a : out std_logic;
      O_processingDone_2b : out std_logic;
      O_processingDone_3 : out std_logic;
      O_Y              : out std_logic_vector(10 downto 0));  -- 11 bit input sample (dont 1 bit de signe)
  end component operativeUnit;

  signal SC_processingDone_1 : std_logic;
  signal SC_processingDone_2a : std_logic;
  signal SC_processingDone_2b : std_logic;
  signal SC_processingDone_3 : std_logic;
  signal SC_loadShift_1      : std_logic;
  signal SC_loadShift_2a      : std_logic;
  signal SC_loadShift_2b      : std_logic;
  signal SC_loadShift_3      : std_logic;
  signal SC_initAddress    : std_logic;
  signal SC_incrAddress    : std_logic;
  signal SC_initSum        : std_logic;
  signal SC_loadSum        : std_logic;
  signal SC_loadY          : std_logic;

begin

  controlUnit_1 : entity work.controlUnit
    port map (
      I_clock               => I_clock,
      I_reset               => I_reset,
      I_inputSampleValid    => I_inputSampleValid,
      I_processingDone_1      => SC_processingDone_1,
      I_processingDone_2a      => SC_processingDone_2a,
      I_processingDone_2b      => SC_processingDone_2b,
      I_processingDone_3      => SC_processingDone_3,
      O_loadShift_1           => SC_loadShift_1,
      O_loadShift_2b           => SC_loadShift_2b,
      O_loadShift_2a           => SC_loadShift_2a,
      O_loadShift_3           => SC_loadShift_3,
      O_initAddress         => SC_initAddress,
      O_incrAddress         => SC_incrAddress,
      O_initSum             => SC_initSum,
      O_loadSum             => SC_loadSum,
      O_loadY               => SC_loadY,
      O_FilteredSampleValid => O_FilteredSampleValid);

  operativeUnit_1 : entity work.operativeUnit
    port map (
      I_clock          => I_clock,
      I_reset          => I_reset,
      I_inputSample    => I_inputSample,
      I_loadShift_1      => SC_loadShift_1,
      I_loadShift_2b      => SC_loadShift_2b,
      I_loadShift_2a      => SC_loadShift_2a,
      I_loadShift_3      => SC_loadShift_3,
      I_initAddress    => SC_initAddress,
      I_incrAddress    => SC_incrAddress,
      I_initSum        => SC_initSum,
      I_loadSum        => SC_loadSum,
      I_loadY          => SC_loadY,
      O_processingDone_1 => SC_processingDone_1,
      O_processingDone_2b => SC_processingDone_2b,
      O_processingDone_2a => SC_processingDone_2a,
      O_processingDone_3 => SC_processingDone_3,
      O_Y              => O_filteredSample);

end architecture archi_firUnit;
